{% extends "base.html" %}

{% block description %}
{{ _("This is the login page. You can log in here.") }}

{% endblock %}

{% block content %}

<div class="login-contents mx-auto w-75">
  <h1>{% block title %}Login{% endblock %}</h1>

  <div class="row align-items-center">

{% for method in login_methods %}
        {{ method() }}

{% endfor %}

  </div>
</div>

{% endblock %}
